*=============== PROGRAM: PQI09_AREA_OBSERVED.SAS =====================;
*
*  Title:  PROGRAM PQI09_AREA_OBSERVED CALCULATES OBSERVED AREA RATES 
*          FOR AHRQ PATIENT QUALITY INDICATOR PQI 09, LOW BIRTH WEIGHT RATE
*
*  Description:
*         USES PROC SUMMARY TO CALCULATE OBSERVED RATES FOR
*         PREVENTION QUALITY INDICATORS ACROSS STRATIFIERS.
*         OUTPUT STRATIFIED BY AREA, AGECAT, SEXCAT AND RACECAT FROM 
*         POPULATION FILE AND PQI09_ALL_MEASURE OUTPUT.
*         VARIABLES CREATED BY THIS PROGRAM ARE PAPQ09 and OAPQ09
*         THIS INDICATOR IS ALSO INCLUDED IN THE PEDIATRIC  
*         QUALITY INDICATOR MODULE.
*
*             >>> AHRQ QI ICD-10 VERSION 6.0.1 <<<
*		        >>> VERSION DATE SEPT, 2016 <<<
*====================================================================;

*PATHNAME specifies the location of the PQI09 folder which includes 
          Programs, SASdata, and Macros subfolders;
%LET PATHNAME=c:\pathname;                        *<==USER MUST modify;
filename CONTROL "&PATHNAME.\PQI09\Programs\PQI09_ALL_CONTROL.SAS"; 

%include CONTROL;

/*MACRO TO ADJUST AREA AGGREGATION BASED ON MAREA VALUE IN PQI09_ALL_CONTROL.*/
 %MACRO CTY2MA;
    %IF &MALEVL EQ 0 %THEN %DO;
        attrib MAREA length=$5
          label='FIPS STATE COUNTY CODE';
        MAREA = FIPSTCO;
    %END;
    %ELSE %IF &MALEVL EQ 1 %THEN %DO;
        attrib MAREA length=$5
          label='MODIFIED FIPS';
        MAREA = input(put(FIPSTCO,$M1AREA.),$5.);
    %END;
    %ELSE %IF &MALEVL EQ 2 %THEN %DO;
        attrib MAREA length=$5
          label='OMB 1999 METRO AREA';
        MAREA = input(put(FIPSTCO,$M2AREA.),$5.);
    %END;
    %ELSE %IF &MALEVL EQ 3 %THEN %DO;
        attrib MAREA length=$5
          label='OMB 2003 METRO AREA';
        MAREA = input(put(FIPSTCO,$M3AREA.),$5.);
    %END;
 %MEND;

 title2 'PROGRAM PQI09_AREA_OBSERVED PART I';
 title3 'AHRQ PREVENTION QUALITY INDICATOR 09: CALCULATE ADJUSTED AREA RATES';

 * ---------------------------------------------------------------- ;
 * --- ADD POPULATION DENOMINATOR --------------------------------- ;
 * --- THIS STEP DETERMINES WHICH AREAS ARE INCLUDED in THE     --- ;
 * --- OUTPUT FROM PQI09_ALL_MEASURES OUTPUT.                   --- ;
 * ---------------------------------------------------------------- ;

 data   TEMP0 (keep=MAREA);
 set    OUTMSR.&OUTFILE_MEAS.;

 %CTY2MA

 run;
/* IDENTIFY UNIQUE MAREA VALUES IN PQI09_ALL_MEASURES OUTPUT. */
 proc   Sort data=TEMP0 (keep=MAREA) out=MAREA nodupkey;
 by     MAREA;
 run;
/* LOAD POPULATION FOR MAREA LEVEL AND YEAR IN PQ09_ALL_CONTROL. */
 data QIPOP0;
    length FIPSTCO $5 SEXCAT POPCAT RACECAT 3 
           POP_1995 POP_1996 POP_1997 POP_1998 POP_1999
           POP_2000 POP_2001 POP_2002 POP_2003 POP_2004
           POP_2005 POP_2006 POP_2007 POP_2008 POP_2009 
           POP_2010 POP_2011 POP_2012 POP_2013 POP_2014 POP_2015
           POP 8;

    infile POPFILE missover firstobs=2;

    input FIPSTCO SEXCAT POPCAT RACECAT 
          POP_1995 POP_1996 POP_1997 POP_1998 POP_1999
          POP_2000 POP_2001 POP_2002 POP_2003 POP_2004
          POP_2005 POP_2006 POP_2007 POP_2008 POP_2009
          POP_2010 POP_2011 POP_2012 POP_2013 POP_2014 POP_2015;

    %CTY2MA

    POP = POP_&POPYEAR.;

    if POPCAT in (1,2,3,4);

 run;

 proc   Summary data=QIPOP0 nway;
 class  MAREA POPCAT SEXCAT RACECAT;
 var    POP;
 output out=QIPOP sum=;
 run;

 proc   Sort data=QIPOP;
 by     MAREA POPCAT SEXCAT RACECAT;
 run;
* LIMIT POPULATION TOTALS TO MAREA CODES FOUND IN PQI09_ALL_MEASURES OUTPUT;
 data   QIPOP(keep=MAREA POPCAT SEXCAT RACECAT POP);
 merge  MAREA(in=X) QIPOP(in=Y);
 by     MAREA;

 if X and Y;

 run;

 * ---------------------------------------------------------------- ;
 * --- PREVENTION QUALITY INDICATOR 09 ADJUSTED RATES ------------- ;
 * ---------------------------------------------------------------- ;
 * AREA-LEVEL INDICATOR DENOMINATORS ARE ADJUSTED BASED ON THE      *
 * COMBINATION OF COUNTY, AGE, SEX, AND RACE IN THE NUMERATOR.   *
 * THIS STEP ITERATES THROUGH THE OUTPUT FROM PQI09_ALL_MEASURES FOR*
 * EACH MEASURE AND REDUCES THE POPULATION SUM BY THE NUMERATOR     *
 * TOTAL. THE MOD3 MACRO ADJUSTS THE RESIDENTIAL POPULATION TOTAL   *
 * BASED ON THE MEASURE RELAVENT POPULATION. ONLY VALID AREA        *
 * CODES ARE RETURNED.                                              *
 * ---------------------------------------------------------------- ;


 %MACRO MOD3(N,PD,DL);

 /* CREATE TEMP TABLE WITH ALL DISCHARGES IN NUMERATOR FOR MEASURE N. */ 

 %IF &DL. = 0 %THEN %DO;

 data   TEMP_2;
 set  OUTMSR.&OUTFILE_MEAS.(keep=KEY FIPSTCO T&PD. POPCAT SEXCAT RACECAT);

 if T&PD. in (1);
 if POPCAT in (1,2,3,4);

 %CTY2MA

 run;

 /* SUM THE NUMERATOR 'T' FLAGS BY MAREA POPCAT AGECAT SEXCAT RACECAT. */
 proc   Summary data=TEMP_2 nway;
 class  MAREA POPCAT SEXCAT RACECAT;
 var    T&PD.;
 output out=TEMP_3 N=TCOUNT;
 run;

 proc   Sort data=TEMP_3;
 by     MAREA POPCAT SEXCAT RACECAT;
 run;

 /* REDUCE THE DENOMINATOR POPULATION BY THE SUM OF THE NUMERATOR COUNT. */
 data   TEMP_4(drop=TCOUNT N);
 merge  QIPOP(in=X keep=MAREA POPCAT SEXCAT RACECAT POP) 
        TEMP_3(keep=MAREA POPCAT SEXCAT RACECAT TCOUNT);
 by     MAREA POPCAT SEXCAT RACECAT;

 if X;

 N = &N.;

 if POPCAT in (1) then do;
    if N = 14 then POP = POP * 0.60; /* AGE < 2 */
    if N = 15 then POP = .;          /* AGE < 6 */
    if N = 16 then POP = POP * 0.95; /* AGEDAY < 90 */
    if N = 18 then POP = POP * 0.95; /* AGEDAY < 90 */
    if N = 90 then POP = .;          /* AGE < 6 */
    if N = 91 then POP = .;          /* AGE < 6 */
    if N = 92 then POP = .;          /* AGE < 6 */
 end;
 else if POPCAT in (2) then do;
   if N = 15 then POP = POP * 0.80; /* AGE < 6 */
   if N = 90 then POP = POP * 0.80; /* AGE < 6 */
   if N = 91 then POP = POP * 0.80; /* AGE < 6 */
   if N = 92 then POP = POP * 0.80; /* AGE < 6 */
 end;

 if TCOUNT > 0 then PCOUNT = POP - TCOUNT;
 else PCOUNT = POP;

 if PCOUNT < 0 then PCOUNT = 0;

 if PCOUNT = 0 then delete;
 
 run;

/* FOR NUMERATOR, RETAIN ONLY RECORDS WITH A VALID MAREA CODE. */
 data   TEMP_3(drop=POP);
 merge  TEMP_3(in=X keep=MAREA POPCAT SEXCAT RACECAT TCOUNT)
        QIPOP(keep=MAREA POPCAT SEXCAT RACECAT POP);
 by     MAREA POPCAT SEXCAT RACECAT;

 if X;

 if POP < 0 then PCOUNT = 0;
 else if TCOUNT > 0 then PCOUNT = TCOUNT;
 else PCOUNT = 0;

 if PCOUNT = 0 then delete;
 
 run;

 /* COMBINE THE NUMERATOR AND DENOMINATOR */;

 data   TEMP1;
 set    TEMP_3(in=X) TEMP_4;

 if X then T&PD. = 1;
 else T&PD. = 0;

 run;

 %END;
 %ELSE %DO;

 data   TEMP_2;
 set    OUTMSR.&OUTFILE_MEAS.(keep=KEY FIPSTCO T&PD. POPCAT SEXCAT RACECAT);

 if T&PD. in (0,1);
 if POPCAT in (1,2,3,4);

 %CTY2MA

 run;

 proc   Summary data=TEMP_2 nway;
 class  T&PD. MAREA POPCAT SEXCAT RACECAT;
 var    T&PD.;
 output out=TEMP_3 N=TCOUNT;
 run;

 proc   Sort data=TEMP_3;
 by     MAREA POPCAT SEXCAT RACECAT;
 run;

 /* RETAIN ONLY RECORDS WITH A VALID MAREA CODE. */
 data   TEMP1;
 merge  TEMP_3(in=X keep=MAREA POPCAT SEXCAT RACECAT TCOUNT T&PD.)
        QIPOP(keep=MAREA POPCAT SEXCAT RACECAT);
 by     MAREA POPCAT SEXCAT RACECAT;

 if X;

 if TCOUNT > 0 then PCOUNT = TCOUNT;
 else PCOUNT = 0;

 if PCOUNT = 0 then delete;
 
 run;

 %END;

  /* ASSIGN POVCAT. */
 data   TEMP1;
 length FEMALE AGECAT1-AGECAT4 FAGECAT1-FAGECAT4
        POVCAT1-POVCAT10 3;
 set    TEMP1;

 if SEXCAT in (2) then FEMALE = 1;
 else FEMALE = 0;

 array ARRY1{4} AGECAT1-AGECAT4;
 array ARRY2{4} FAGECAT1-FAGECAT4;

 do I = 1 TO 4;
    ARRY1(I) = 0; ARRY2(I) = 0;
 end;

 ARRY1(POPCAT) = 1;
 ARRY2(POPCAT) = FEMALE;

 array ARRY3{10} POVCAT1-POVCAT10;

 do I = 1 TO 10;
    ARRY3(I) = 0;
 end;

 PVIDX = put(MAREA,$POVCAT.);

 if PVIDX > 0 then ARRY3(PVIDX) = 1;

 run;

 
 data   TEMP1Y;
 set    TEMP1;

 ONE = 1;

 run;

 proc   Summary data=TEMP1Y;
 class  MAREA POPCAT SEXCAT RACECAT;
 var    T&PD. ONE;
 output out=ADJ_&PD. sum(T&PD. ONE)=T&PD. P&PD.;
 weight PCOUNT;
 run;

 data   ADJ_&PD.(keep=MAREA POPCAT SEXCAT RACECAT _TYPE_ T&PD. P&PD.);
 set    ADJ_&PD.;

 if _TYPE_ &TYPELVLA;

 run;

 proc Sort data=ADJ_&PD.;
     by MAREA POPCAT SEXCAT RACECAT;
 run; quit;

 proc   Datasets nolist;
 delete TEMP1 TEMP1Y TEMP_2 TEMP_3 TEMP_4;
 run;

 %MEND;

 %MOD3(60,APQ09,1);

/* MERGE THE ADJUSTED DENOMINATOR AND NUMERATOR FOR AREA PREVENTION QUALITY INDICATOR 09. */
data TEMP2Y;
  set ADJ_APQ09;
  by MAREA POPCAT  SEXCAT RACECAT;
run;

 * --------------------------------------------------------------- ;
* --- PREVENTION QUALITY INDICATOR (PQI09) NAMING CONVENTION:  --- ;
* --- THE FIRST LETTER IDENTIFIES THE PREVENTION QUALITY       --- ;
* --- INDICATOR AS ONE OF THE FOLLOWING:
                (T) NUMERATOR ("TOP")
                (P) DENOMINATOR ("POPULATION")
                (O) OBSERVED RATES
* --- THE SECOND LETTER REFERS TO THE INDICATOR                --- ;
* --- SUBTYPE, (A)REA. THE NEXT TWO CHARACTERS                 --- ;
* --- ARE ALWAYS 'PQ'. THE LAST TWO DIGITS ARE                 --- ;
* --- THE INDICATOR NUMBER (09).                               --- ;
* ---------------------------------------------------------------- ;

/* CALCULATE OBSERVED RATE AS SUM OF NUMERATOR / SUM OF ADJUSTED DENOMINATOR.*/ 
data &OUTFILE_AREAOBS.; 
 set TEMP2Y;

 array PPD2{1} PAPQ09;
 array TPD{1}  TAPQ09;
 array OPD{1}  OAPQ09;

 do J = 1 to 1;
    if TPD{J} GT 0 and PPD2{J} GT 0 then OPD{J} = TPD{J} / PPD2{J};
    else if PPD2{J} GT 0 then OPD{J} = 0;
 end;

 label
 TAPQ09 = 'PQI 09 Low Birth Weight Rate (Numerator)'
 ;
 label
 PAPQ09 = 'PQI 09 Low Birth Weight Rate (Population)'
 ;
 label
 OAPQ09 = 'PQI 09 Low Birth Weight Rate (Observed)'
 _TYPE_ = 'STRATIFICATION LEVEL'
 ;

 drop J;

 run;

 proc Sort data=&OUTFILE_AREAOBS. out=OUTAOBS.&OUTFILE_AREAOBS.;
 by MAREA POPCAT SEXCAT RACECAT;
 run;

proc Datasets nolist;
  delete MAREA QIPOP QIPOP0 TEMP0 TEMP2Y
         ADJ_APQ09;
run; quit;

 * ------------------------------------------------------ ;
* --- CONTENTS AND MEANS OF AREA OBSERVED OUTPUT FILE --- ;
 * ------------------------------------------------------ ;

 proc Contents data=OUTAOBS.&OUTFILE_AREAOBS. position;
 run;

 *proc Means data=OUTAOBS.&OUTFILE_AREAOBS.(where=(_TYPE_ in (8))) n nmiss min max mean sum;
 *title4 "SUMMARY OF AREA-LEVEL RATES (_TYPE_=8)";
 *run;

***----- TO PRINT VARIABLE LABELS COMMENT (DELETE) "NOLABELS" FROM PROC MEANS STATEMENTS -------***;

proc Means data=OUTAOBS.&OUTFILE_AREAOBS. (where=(_TYPE_ in (8))) n nmiss min max sum nolabels;
   var TAPQ09 ;
   title  'SUMMARY OF PREVENTION QUALITY AREA-LEVEL INDICATOR OVERALL NUMERATOR (SUM) WHEN _TYPE_=8';
run; quit;

proc Means data=OUTAOBS.&OUTFILE_AREAOBS. (where=(_TYPE_ in (8))) n nmiss min max sum nolabels;
   var PAPQ09;
   title  'SUMMARY OF PREVENTION QUALITY AREA-LEVEL INDICATOR OVERALL DENOMINATOR (SUM) WHEN _TYPE_=8';
run; quit;

proc Means data=OUTAOBS.&OUTFILE_AREAOBS. (where=(_TYPE_ in (8))) n nmiss min max mean nolabels;
   var OAPQ09 ;
   title  'SUMMARY OF PREVENTION QUALITY AREA-LEVEL INDICATOR AVERAGE RATES(MEAN) WHEN _TYPE_=8';
run; quit;

 * -------------------------------------------------------------- ;
 * --- PRINT AREA OBSERVED MEANS FILE TO SAS OUTPUT           --- ;
 * -------------------------------------------------------------- ;

 %MACRO PRT2;

 %IF &PRINT. = 1 %THEN %DO;

 %MACRO PRT(PD,TEXT);

 proc  PRINT data=OUTAOBS.&OUTFILE_AREAOBS. label SPLIT='*';
 var   MAREA POPCAT SEXCAT RACECAT TA&PD. PA&PD. OA&PD. ;
 label MAREA    = "MAREA"
       POPCAT   = "POPCAT"
       SEXCAT   = "SEXCAT"
       RACECAT  = "RACECAT"
       TA&PD.   = "TA&PD.*(Numerator)"
       PA&PD.   = "PA&PD.*(Denominator)"
       OA&PD.   = "OA&PD.*(Observed)"
       ;
 format POPCAT POPCAT.   
        SEXCAT SEXCAT.
        RACECAT RACECAT.
        TA&PD. PA&PD. COMMA13.0
        OA&PD. 8.6
        ;
 title4 "Indicator &PD.: &TEXT";
 run;

 %MEND PRT;

 %PRT(PQ09,Low Birth Weight Rate);

 %END;

 %MEND PRT2;

 %PRT2; 

 * -------------------------------------------------------------- ;
 * --- WRITE SAS OUTPUT DATA SET TO COMMA-DELIMITED TEXT FILE --- ;
 * --- FOR EXPORT INTO SPREADSHEETS ----------------------------- ;
 * -------------------------------------------------------------- ;

 %MACRO TEXT;
 %IF &TXTAOBS. = 1  %THEN %DO;
	%LET TYPEARN  = %sysfunc(tranwrd(&TYPELVLA.,%str(,),_));
	%LET TYPEARN2  = %sysfunc(compress(&TYPEARN.,'(IN )'));
	%let PRCSVREF = %sysfunc(pathname(P9TXTAOB));
	%let PDCSVRF2 =  %sysfunc(tranwrd(&PRCSVREF.,.,_&TYPEARN2..));
 data _NULL_;
 set OUTAOBS.&OUTFILE_AREAOBS.;
 file "&PDCSVRF2." lrecl=1000;
  if _N_=1 then put "MAREA"  "," "Age"  "," "Sex"  "," "Race"  "," "Type" ","
 "TAPQ09" "," 
 "PAPQ09" "," 
 "OAPQ09"  
;

 put MAREA $5. "," POPCAT 3. "," SEXCAT 3. "," RACECAT 3.  "," _TYPE_ 2.  ","
 (TAPQ09) (7.0 ",")
 ","
 (PAPQ09) (13.2 ",")
 ","
 (OAPQ09) (12.10 ",")
 ;
 run;

 %END;

 %MEND TEXT;

 %TEXT;
